using System;
using System.IO;
using System.Collections.Generic;
using System.Xml;
using System.Linq;
using System.Text.RegularExpressions;
using System.Data.SQLite;
using System.Globalization;

namespace PandoraBox
{
    public class WordpressMacrosPlugin : IPluginMacro
    {
        const string VERSION = "WPX0.4";
        
        //#Include:System.Data.dll
        //#Include:System.Data.SQLite.dll
        public string Execute(string template, PluginMacroArgs args)
        {
            SQLHandler handler = args.Context["sql"] as SQLHandler;
            if (handler == null)
                args.Context["sql"] = handler = new SQLHandler(args);

            String fileName = args.Key;

            if (fileName == "finish")
            {
                handler.Transaction.Commit();
                DelEmptyDir(args.Folder);
                DelFile(args.Folder + "/delayeddump.php");
                DelFile(args.Folder + "/delayedlinks.php");
                AppendImgRewrite(args.Folder + "/.htaccess");
                DelFile(args.Folder + "/.htaccess");
            }
            else if (fileName.EndsWith(handler.Extension))
            {
                var post = new PostTemplate(template);
                if (!post.Skip) 
                    handler.Post(post);
                else if (template.Contains("{IBTITLE}"))
                    UpdateInstall(template, args);
            }

            return "OK";
        }
        
        private void UpdateInstall(string template, PluginMacroArgs args)
        {
            var fi = new FileInfo(args.Folder + "/install.php");
            if (fi.Exists) 
            {
                var install = File.ReadAllText(fi.FullName);
                install = install.Replace("[IBTITLE]", Regex.Match(template, "{IBTITLE}(.*?){/IBTITLE}", RegexOptions.Singleline).Groups[1].Value.Replace("\"","").Replace("'","").Trim());
                install = install.Replace("[IBDESCRIPTION]", Regex.Match(template, "{IBDESCRIPTION}(.*?){/IBDESCRIPTION}", RegexOptions.Singleline).Groups[1].Value.Replace("\"","").Replace("'","").Trim());
                File.WriteAllText(fi.FullName, install);
            }
        }

        private void DelEmptyDir(string startLocation)
        {
            foreach (var directory in Directory.GetDirectories(startLocation))
            {
                DelEmptyDir(directory);
                if (Directory.GetFiles(directory).Length == 0 &&
                    Directory.GetDirectories(directory).Length == 0)
                {
                    Directory.Delete(directory, false);
                }
            }
        }

        private void DelFile(string file)
        {
            if (File.Exists(file)) File.Delete(file);
        }
        
        private void AppendImgRewrite(string file)
        {
            if (!File.Exists(file)) return;
            var rule = File.ReadAllLines(file).FirstOrDefault(x => x.Contains("proxyimg"));
            if (rule != null) File.WriteAllText(file + ".img", rule + "\n\n");
        }

        public ushort Level { get { return 220; } }
        public String Name { get { return "WordpressMacrosPlugin"; } }
    }

    class SQLHandler
    {
        public String Extension;
        public String Host;

        public SQLiteConnection Connection;
        public SQLiteTransaction Transaction;

        private int _postsCounter;
        private SQLiteCommand _wp_posts;
        private SQLiteCommand _wp_term_relationships;
        private int _taxonomyCounter;
        private SQLiteCommand _wp_term_taxonomy;
        private int _termCounter;
        private SQLiteCommand _wp_terms;
        private Dictionary<string, int> _terms = new Dictionary<string, int>();
        private Dictionary<int, Taxonomy> _taxonomy = new Dictionary<int, Taxonomy>();

        public SQLHandler(PluginMacroArgs args)
        {
            File.Copy("Plugins/wpx-db.db", args.Folder + "/db.db", true);
            Connection = new SQLiteConnection("Data Source = \"" + args.Folder + "/db.db\"");
            Connection.Open();
            XmlDocument profile = args.Context["$profile"] as XmlDocument;
            Extension = profile.SelectSingleNode("//FilesExt").InnerText;
            Host = profile.SelectSingleNode("//Door/Url").InnerText;
            new SQLiteCommand("delete from wp_posts", Connection).ExecuteNonQuery();
            
            Transaction = Connection.BeginTransaction();

            _wp_posts = new SQLiteCommand("insert into wp_posts (ID, post_author, post_date, post_date_gmt, post_content, post_title, post_excerpt, post_status, comment_status, ping_status, post_password, post_name, to_ping, pinged, post_modified, post_modified_gmt, post_content_filtered, post_parent, guid, menu_order, post_type, post_mime_type, comment_count) values (:ID, :post_author, :post_date, :post_date_gmt, :post_content, :post_title, :post_excerpt, :post_status, :comment_status, :ping_status, :post_password, :post_name, :to_ping, :pinged, :post_modified, :post_modified_gmt, :post_content_filtered, :post_parent, :guid, :menu_order, :post_type, :post_mime_type, :comment_count)", Connection, Transaction);
            _wp_term_relationships = new SQLiteCommand("insert into wp_term_relationships (object_id, term_taxonomy_id, term_order) values (:object_id, :term_taxonomy_id, :term_order)", Connection, Transaction);
            _wp_term_taxonomy = new SQLiteCommand("insert into wp_term_taxonomy (term_taxonomy_id, term_id, taxonomy, description, parent, count) values (:term_taxonomy_id, :term_id, :taxonomy, :description, :parent, :count)", Connection, Transaction);
            _wp_terms = new SQLiteCommand("insert into wp_terms (term_id, name, slug, term_group) values (:term_id, :name, :slug, :term_group)", Connection, Transaction);
            AddParams(_wp_posts);
            AddParams(_wp_term_relationships);
            AddParams(_wp_term_taxonomy);
            AddParams(_wp_terms);
        }

        private void AddParams(SQLiteCommand cmd)
        {
            foreach (Match m in Regex.Matches(cmd.CommandText, ":(\\w+)"))
                cmd.Parameters.Add(new SQLiteParameter(m.Groups[1].Value, ""));
        }

        public void Post(PostTemplate post)
        {
            var postId = AddPost(post);

            foreach (var cat in post.Category)
            {
                int termId = AddTerm(cat);
                var taxonomyId = AddTaxonomy(termId, "category");
                AddRelation(postId, taxonomyId);
            }

            foreach (var tag in post.Tags)
            {
                if (tag.Length < 3) continue;
                int termId = AddTerm(tag);
                var taxonomyId = AddTaxonomy(termId, "post_tag");
                AddRelation(postId, taxonomyId);
            }
        }

        private int AddPost(PostTemplate post)
        {
            int postId = ++_postsCounter;
            DateTime postTime = DateTime.ParseExact(post.Date, "u", CultureInfo.InvariantCulture);
            post.Date = post.Date.TrimEnd('Z');
            _wp_posts.Parameters["ID"].Value = postId;
            _wp_posts.Parameters["post_author"].Value = 1;
            _wp_posts.Parameters["post_date"].Value = post.Date;
            _wp_posts.Parameters["post_date_gmt"].Value = post.Date;
            _wp_posts.Parameters["post_content"].Value = post.Text;
            _wp_posts.Parameters["post_title"].Value = post.Title;
            _wp_posts.Parameters["post_excerpt"].Value = "";
            _wp_posts.Parameters["post_status"].Value = DateTime.Now > postTime ? "publish" : "future";
            _wp_posts.Parameters["comment_status"].Value = "open";
            _wp_posts.Parameters["ping_status"].Value = "open";
            _wp_posts.Parameters["post_password"].Value = "";
            _wp_posts.Parameters["post_name"].Value = Slug(post.Url);
            _wp_posts.Parameters["to_ping"].Value = "";
            _wp_posts.Parameters["pinged"].Value = "";
            _wp_posts.Parameters["post_modified"].Value = post.Date;
            _wp_posts.Parameters["post_modified_gmt"].Value = post.Date;
            _wp_posts.Parameters["post_content_filtered"].Value = "";
            _wp_posts.Parameters["post_parent"].Value = 0;
            _wp_posts.Parameters["guid"].Value = Host + "?p=" + postId;
            _wp_posts.Parameters["menu_order"].Value = 0;
            _wp_posts.Parameters["post_type"].Value = "post";
            _wp_posts.Parameters["post_mime_type"].Value = "";
            _wp_posts.Parameters["comment_count"].Value = 0;
            _wp_posts.ExecuteNonQuery();
            return postId;
        }

        private void AddRelation(int postId, int taxonomyId)
        {
            _wp_term_relationships.Parameters["object_id"].Value = postId;
            _wp_term_relationships.Parameters["term_taxonomy_id"].Value = taxonomyId;
            _wp_term_relationships.Parameters["term_order"].Value = "0";
            _wp_term_relationships.ExecuteNonQuery();
        }

        class Taxonomy
        {
            public int ID { get; set; }
            public int Count { get; set; }
        }

        private int AddTaxonomy(int termId, string type)
        {
            Taxonomy taxonomy;
            if (!_taxonomy.TryGetValue(termId, out taxonomy))
            {
                taxonomy = new Taxonomy { ID = ++_taxonomyCounter, Count = 1 };
                _taxonomy.Add(termId, taxonomy);
                _wp_term_taxonomy.Parameters["term_taxonomy_id"].Value = taxonomy.ID;
                _wp_term_taxonomy.Parameters["term_id"].Value = termId;
                _wp_term_taxonomy.Parameters["taxonomy"].Value = type;
                _wp_term_taxonomy.Parameters["description"].Value = "";
                _wp_term_taxonomy.Parameters["parent"].Value = 0;
                _wp_term_taxonomy.Parameters["count"].Value = 1;
                _wp_term_taxonomy.ExecuteNonQuery();
            }
            else
                taxonomy.Count++;
            return taxonomy.ID;
        }

        private int AddTerm(string name)
        {
            int termId;
            if (!_terms.TryGetValue(name, out termId))
            {
                termId = ++_termCounter;
                _terms.Add(name, termId);
                _wp_terms.Parameters["term_id"].Value = termId;
                _wp_terms.Parameters["name"].Value = name;
                _wp_terms.Parameters["slug"].Value = Slug(name);
                _wp_terms.Parameters["term_group"].Value = 0;
                _wp_terms.ExecuteNonQuery();
            }
            return termId;
        }

        private string Slug(String name)
        {
            return Uri.EscapeDataString(name.ToLower().Replace(" ", "-")).ToLower();
        }
    }

    public class PostTemplate
    {
        public bool Skip { get; set; }

        public string Url { get; set; }
        public string Title { get; set; }
        public List<string> Category { get; set; }
        public List<string> Tags { get; set; }
        public string Date { get; set; }
        public string Text { get; set; }

        public PostTemplate(String content)
        {
            Title = Parse(content, "IBTITLE");
            if (String.IsNullOrEmpty(Title))
                Skip = true;

            Text = Parse(content, "IBCONT");
            if (String.IsNullOrEmpty(Text))
                Skip = true;

            Category = Parse(content, "IBCATS")
                .Split(',')
                .Select(x => x.Trim())
                .Where(x => !String.IsNullOrEmpty(x))
                .ToList();

            Tags = Parse(content, "IBTAGS")
                .Split(',')
                .Select(x => x.Trim())
                .Where(x => !String.IsNullOrEmpty(x))
                .ToList();

            Date = Parse(content, "IBDATE");
            Url = Parse(content, "IBURL");
        }

        private string Parse(string content, string token)
        {
            return Regex.Match(content, "{" + token + "}(.*?){/" + token + "}", RegexOptions.Singleline).Groups[1].Value.Trim();
        }

        public override string ToString()
        {
            return Title;
        }
    }
}